@@ -676,7 +676,7 @@ QDialog* ConfigWidget::configDialog(QWidget* parent, FcitxConfigFileDesc* cfdesc
         dialog
     );
     dialog->setWindowIcon(QIcon::fromTheme("fcitx"));
-    QHBoxLayout* dialogLayout = new QHBoxLayout;
+    QVBoxLayout* dialogLayout = new QVBoxLayout;
     dialog->setLayout(dialogLayout);
     QDialogButtonBox* buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel | QDialogButtonBox::RestoreDefaults);
     dialogLayout->addWidget(configPage);
@@ -685,6 +685,8 @@ QDialog* ConfigWidget::configDialog(QWidget* parent, FcitxConfigFileDesc* cfdesc
     connect(buttonBox, &QDialogButtonBox::clicked, configPage, [configPage, buttonBox](QAbstractButton* button) {
         configPage->buttonClicked(buttonBox->standardButton(button));
     });
+    connect(buttonBox, &QDialogButtonBox::accepted, dialog, &QDialog::accept);
+    connect(buttonBox, &QDialogButtonBox::rejected, dialog, &QDialog::reject);
 
     return dialog;
 }
