@@ -1,6 +1,6 @@
 #include <fcitx/module/dbus/dbusstuff.h>
 #include <fcitx/module/ipc/ipc.h>
-#include <QHBoxLayout>
+#include <QVBoxLayout>
 #include <QComboBox>
 #include <QLabel>
 #include <KComboBox>
@@ -20,7 +20,7 @@ Fcitx::IMConfigDialog::IMConfigDialog(const QString& imName, const FcitxAddon* a
     ,m_configPage(0)
 {
     QWidget* widget = new QWidget(this);
-    QHBoxLayout* l = new QHBoxLayout(this);
+    QVBoxLayout* l = new QVBoxLayout(this);
     widget->setLayout(l);
 
     if (!imName.startsWith("fcitx-keyboard") && Global::instance()->keyboardProxy()) {
@@ -109,14 +109,16 @@ Fcitx::IMConfigDialog::IMConfigDialog(const QString& imName, const FcitxAddon* a
         }
     }
     setWindowIcon(QIcon::fromTheme("fcitx"));
-    QHBoxLayout* dialogLayout = new QHBoxLayout;
+    auto dialogLayout = new QVBoxLayout;
     setLayout(dialogLayout);
-    QDialogButtonBox* buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel | QDialogButtonBox::RestoreDefaults);
+    auto buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel | QDialogButtonBox::RestoreDefaults);
     dialogLayout->addWidget(widget);
     dialogLayout->addWidget(buttonBox);
     connect(buttonBox, &QDialogButtonBox::clicked, this, [this, buttonBox](QAbstractButton* button) {
         this->onButtonClicked(buttonBox->standardButton(button));
     });
+    connect(buttonBox, &QDialogButtonBox::accepted, this, &Fcitx::IMConfigDialog::accept);
+    connect(buttonBox, &QDialogButtonBox::rejected, this, &Fcitx::IMConfigDialog::reject);
 }
 
 void Fcitx::IMConfigDialog::onButtonClicked(QDialogButtonBox::StandardButton code)
