@@ -25,7 +25,7 @@
 
 // KDE
 #include <KStringHandler>
-#include <KCategoryDrawer>
+#include <KLocalizedString>
 
 #include <fcitxqtinputmethodproxy.h>
 
@@ -44,22 +44,6 @@
 namespace Fcitx
 {
 
-static bool languageIsUnknown(const QString& langCode)
-{
-    if (langCode.isEmpty()) {
-        return true;
-    }
-    else if (langCode == "*")
-        return false;
-    else {
-        const QString result = QLocale(langCode).nativeLanguageName();
-        if (result.isEmpty()) {
-            return true;
-        }
-    }
-    return false;
-}
-
 static QString languageName(const QString& langCode)
 {
     if (langCode.isEmpty()) {
@@ -68,17 +52,198 @@ static QString languageName(const QString& langCode)
     else if (langCode == "*")
         return i18n("Multilingual");
     else {
-        const QString result = QLocale(langCode).nativeLanguageName();
-        if (result.isEmpty()) {
-            return i18n("Unknown");
+        QLocale locale(langCode);
+        if (locale.language() == QLocale::C) {
+            // return lang code seems to be a better solution other than indistinguishable "unknown"
+            return langCode;
+        }
+        const bool hasCountry = langCode.indexOf("_") != -1 && locale.country() != QLocale::AnyCountry;
+        QString languageName;
+        if (hasCountry) {
+            languageName = locale.nativeLanguageName();
+        }
+        if (languageName.isEmpty()) {
+            languageName = i18nd("iso_639", QLocale::languageToString(locale.language()).toUtf8());
+        }
+        if (languageName.isEmpty()) {
+            languageName = i18n("Other");
+        }
+        QString countryName;
+        // QLocale will always assign a default country for us, check if our lang code
+
+        if (langCode.indexOf("_") != -1 && locale.country() != QLocale::AnyCountry) {
+            countryName = locale.nativeCountryName();
+            if (countryName.isEmpty()) {
+                countryName = QLocale::countryToString(locale.country());
+            }
+        }
+
+        if (countryName.isEmpty()) {
+            return languageName;
+        } else {
+            return i18nc("%1 is language name, %2 is country name", "%1 (%2)", languageName, countryName);
         }
-        return result;
     }
 }
 
-IMPage::Private::IMModel::IMModel(bool filterEnabled_, QObject* parent)
-    : QAbstractListModel(parent),
-      filterEnabled(filterEnabled_)
+IMPage::Private::AvailIMModel::AvailIMModel(QObject* parent)
+    : QAbstractItemModel(parent)
+{
+}
+
+IMPage::Private::AvailIMModel::~AvailIMModel()
+{
+}
+
+QModelIndex IMPage::Private::AvailIMModel::index(int row, int column, const QModelIndex& parent) const
+{
+    // return language index
+    if (!parent.isValid()) {
+        if (column > 0 || row >= filteredIMEntryList.count()) {
+            return QModelIndex();
+        } else {
+            return createIndex(row, column, static_cast<quintptr>(0));
+        }
+    }
+
+    // return im index
+    if (parent.column() > 0 || parent.row() >= filteredIMEntryList.count() ||
+        row >= filteredIMEntryList[parent.row()].second.size()) {
+        return QModelIndex();
+    }
+
+    return createIndex(row, column, parent.row() + 1);
+}
+
+QVariant IMPage::Private::AvailIMModel::data(const QModelIndex& index, int role) const
+{
+    if (!index.isValid()) {
+        return QVariant();
+    }
+
+    if (!index.parent().isValid()) {
+        if (index.column() > 0 || index.row() >= filteredIMEntryList.count()) {
+            return QVariant();
+        }
+        switch (role) {
+
+        case Qt::DisplayRole:
+            return languageName(filteredIMEntryList[index.row()].first);
+
+        case FcitxLanguageRole:
+            return filteredIMEntryList[index.row()].first;
+
+        case FcitxIMUniqueNameRole:
+            return QString();
+
+        case FcitxRowTypeRole:
+            return LanguageType;
+
+        default:
+            return QVariant();
+        }
+    }
+
+    if (index.column() > 0 || index.parent().column() > 0 || index.parent().row() >= filteredIMEntryList.count()) {
+        return QVariant();
+    }
+
+    const FcitxQtInputMethodItemList& imEntryList = filteredIMEntryList[index.parent().row()].second;
+
+    if (index.row() >= imEntryList.count()) {
+        return QVariant();
+    }
+
+    const FcitxQtInputMethodItem& imEntry = imEntryList[index.row()];
+
+    switch (role) {
+
+    case Qt::DisplayRole:
+        return imEntry.name();
+
+    case FcitxRowTypeRole:
+        return IMType;
+
+    case FcitxIMUniqueNameRole:
+        return imEntry.uniqueName();
+
+    case FcitxLanguageRole:
+        return imEntry.langCode();
+    }
+    return QVariant();
+}
+
+QModelIndex IMPage::Private::AvailIMModel::parent(const QModelIndex& child) const
+{
+    if (!child.isValid()) {
+        return QModelIndex();
+    }
+
+    auto row = child.internalId();
+    if (row && row - 1 >= filteredIMEntryList.count()) {
+        return QModelIndex();
+    }
+
+    return createIndex(row - 1, 0, -1);
+}
+
+int IMPage::Private::AvailIMModel::rowCount(const QModelIndex& parent) const
+{
+    if (!parent.isValid()) {
+        return filteredIMEntryList.count();
+    }
+
+    if (parent.internalId() > 0) {
+        return 0;
+    }
+
+    if (parent.column() > 0 || parent.row() >= filteredIMEntryList.count()) {
+        return 0;
+    }
+
+    return filteredIMEntryList[parent.row()].second.count();
+}
+
+int IMPage::Private::AvailIMModel::columnCount(const QModelIndex& parent) const
+{
+    Q_UNUSED(parent);
+    return 1;
+}
+
+void IMPage::Private::AvailIMModel::filterIMEntryList(const FcitxQtInputMethodItemList& imEntryList, const QString& selection)
+{
+    beginResetModel();
+
+    QMap<QString, int> languageMap;
+    filteredIMEntryList.clear();
+    int langRow = -1;
+    int imRow = -1;
+    Q_FOREACH(const FcitxQtInputMethodItem & im, imEntryList) {
+        if (!im.enabled()) {
+            int idx;
+            if (!languageMap.contains(im.langCode())) {
+                idx = filteredIMEntryList.count();
+                languageMap[im.langCode()] = idx;
+                filteredIMEntryList.append(QPair<QString, FcitxQtInputMethodItemList>(im.langCode(), FcitxQtInputMethodItemList()));
+            } else {
+                idx = languageMap[im.langCode()];
+            }
+            filteredIMEntryList[idx].second.append(im);
+            if (im.uniqueName() == selection) {
+                langRow = idx;
+                imRow = filteredIMEntryList[idx].second.count() - 1;
+            }
+        }
+    }
+    endResetModel();
+
+    if (imRow >= 0) {
+        emit select(index(imRow, 0, index(langRow, 0)));
+    }
+}
+
+IMPage::Private::IMModel::IMModel(QObject* parent)
+    : QAbstractListModel(parent)
 {
 }
 
@@ -106,15 +271,15 @@ QVariant IMPage::Private::IMModel::data(const QModelIndex& index, int role) cons
     case Qt::DisplayRole:
         return imEntry.name();
 
-    case Qt::UserRole:
+    case FcitxRowTypeRole:
+        return IMType;
+
+    case FcitxIMUniqueNameRole:
         return imEntry.uniqueName();
 
-    case KCategorizedSortFilterProxyModel::CategorySortRole:
+    case FcitxLanguageRole:
         return imEntry.langCode();
 
-    case KCategorizedSortFilterProxyModel::CategoryDisplayRole:
-        return QLocale(imEntry.langCode()).nativeLanguageName();
-
     default:
         return QVariant();
     }
@@ -137,7 +302,7 @@ void IMPage::Private::IMModel::filterIMEntryList(const FcitxQtInputMethodItemLis
     filteredIMEntryList.clear();
     int row = 0, selectionRow = -1;
     Q_FOREACH(const FcitxQtInputMethodItem & im, imEntryList) {
-        if (im.enabled() ^ filterEnabled) {
+        if (im.enabled()) {
             filteredIMEntryList.append(im);
             if (im.uniqueName() == selection)
                 selectionRow = row;
@@ -155,7 +320,7 @@ void IMPage::Private::IMModel::filterIMEntryList(const FcitxQtInputMethodItemLis
 }
 
 IMPage::Private::IMProxyModel::IMProxyModel(QAbstractItemModel* sourceModel)
-    : KCategorizedSortFilterProxyModel(sourceModel)
+    : QSortFilterProxyModel(sourceModel)
      ,m_showOnlyCurrentLanguage(true)
 {
 }
@@ -190,7 +355,6 @@ void IMPage::Private::IMProxyModel::filterIMEntryList(const FcitxQtInputMethodIt
             m_languageSet.insert(im.langCode().left(2));
         }
     }
-    sort(0);
     invalidate();
 }
 
@@ -199,14 +363,37 @@ bool IMPage::Private::IMProxyModel::filterAcceptsRow(int source_row, const QMode
 {
     const QModelIndex index = sourceModel()->index(source_row, 0, source_parent);
 
+    if (index.data(FcitxRowTypeRole) == LanguageType) {
+        return filterLanguage(index);
+    }
+
     return filterIM(index);
 }
 
+bool IMPage::Private::IMProxyModel::filterLanguage(const QModelIndex& index) const
+{
+    if (!index.isValid()) {
+        return false;
+    }
+
+    int childCount = index.model()->rowCount(index);
+    if (childCount == 0)
+        return false;
+
+    for (int i = 0; i < childCount; ++i) {
+        if (filterIM(index.model()->index(i, 0, index))) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
 bool IMPage::Private::IMProxyModel::filterIM(const QModelIndex& index) const
 {
-    QString uniqueName = index.data(Qt::UserRole).toString();
+    QString uniqueName = index.data(FcitxIMUniqueNameRole).toString();
     QString name = index.data(Qt::DisplayRole).toString();
-    QString langCode = index.data(KCategorizedSortFilterProxyModel::CategorySortRole).toString();
+    QString langCode = index.data(FcitxLanguageRole).toString();
 
     if (uniqueName == "fcitx-keyboard-us")
         return true;
@@ -227,7 +414,7 @@ bool IMPage::Private::IMProxyModel::filterIM(const QModelIndex& index) const
     return flag;
 }
 
-bool IMPage::Private::IMProxyModel::subLessThan(const QModelIndex& left, const QModelIndex& right) const
+bool IMPage::Private::IMProxyModel::lessThan(const QModelIndex& left, const QModelIndex& right) const
 {
     int result = compareCategories(left, right);
     if (result < 0) {
@@ -243,8 +430,8 @@ bool IMPage::Private::IMProxyModel::subLessThan(const QModelIndex& left, const Q
 
 int IMPage::Private::IMProxyModel::compareCategories(const QModelIndex& left, const QModelIndex& right) const
 {
-    QString l = left.data(CategorySortRole).toString();
-    QString r = right.data(CategorySortRole).toString();
+    QString l = left.data(FcitxLanguageRole).toString();
+    QString r = right.data(FcitxLanguageRole).toString();
 
     if (l == r)
         return 0;
@@ -293,18 +480,15 @@ IMPage::IMPage(Module* parent): QWidget(parent)
     d->filterTextEdit->setClearButtonEnabled(true);
     d->filterTextEdit->setPlaceholderText(i18n("Search Input Method"));
 
-    d->availIMModel = new Private::IMModel(true, d);
+    d->availIMModel = new Private::AvailIMModel(d);
     connect(d, SIGNAL(updateIMList(FcitxQtInputMethodItemList,QString)), d->availIMModel, SLOT(filterIMEntryList(FcitxQtInputMethodItemList,QString)));
     d->availIMProxyModel = new Private::IMProxyModel(d->availIMModel);
     d->availIMProxyModel->setSourceModel(d->availIMModel);
     connect(d, SIGNAL(updateIMList(FcitxQtInputMethodItemList,QString)), d->availIMProxyModel, SLOT(filterIMEntryList(FcitxQtInputMethodItemList,QString)));
-    d->availIMProxyModel->setCategorizedModel(true);
+    d->availIMView->setItemDelegate(new IMDelegate);
     d->availIMView->setModel(d->availIMProxyModel);
-    d->availIMView->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
-    d->availIMView->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
-    d->availIMView->setCategoryDrawer(new KCategoryDrawer(d->availIMView));
 
-    d->currentIMModel = new Private::IMModel(false, this);
+    d->currentIMModel = new Private::IMModel(this);
     connect(d, SIGNAL(updateIMList(FcitxQtInputMethodItemList,QString)), d->currentIMModel, SLOT(filterIMEntryList(FcitxQtInputMethodItemList,QString)));
     d->currentIMView->setModel(d->currentIMModel);
     d->currentIMView->setSelectionMode(QAbstractItemView::SingleSelection);
@@ -320,6 +504,7 @@ IMPage::IMPage(Module* parent): QWidget(parent)
     connect(d->configureButton, SIGNAL(clicked(bool)), d, SLOT(configureIM()));
     connect(d, SIGNAL(changed()), this, SIGNAL(changed()));
     connect(d->availIMModel, SIGNAL(select(QModelIndex)), d, SLOT(selectAvailIM(QModelIndex)));
+    connect(d->availIMProxyModel, SIGNAL(layoutChanged()), d->availIMView, SLOT(expandAll()));
     connect(d->currentIMModel, SIGNAL(select(QModelIndex)), d, SLOT(selectCurrentIM(QModelIndex)));
     connect(d->defaultLayoutButton, SIGNAL(clicked(bool)), d, SLOT(selectDefaultLayout()));
     connect(d->availIMView, SIGNAL(doubleClicked(QModelIndex)), d, SLOT(doubleClickAvailIM(QModelIndex)));
@@ -444,7 +629,7 @@ void IMPage::Private::clickRemoveIM()
 void IMPage::Private::addIM(const QModelIndex& index)
 {
     if (index.isValid()) {
-        const QString uniqueName =index.data(Qt::UserRole).toString();
+        const QString uniqueName =index.data(FcitxIMUniqueNameRole).toString();
         for (int i = 0; i < m_list.size(); i ++) {
             if (uniqueName == m_list[i].uniqueName()) {
                 m_list[i].setEnabled(true);
@@ -460,7 +645,7 @@ void IMPage::Private::addIM(const QModelIndex& index)
 void IMPage::Private::removeIM(const QModelIndex& index)
 {
     if (index.isValid()) {
-        const QString uniqueName =index.data(Qt::UserRole).toString();
+        const QString uniqueName =index.data(FcitxIMUniqueNameRole).toString();
         for (int i = 0; i < m_list.size(); i ++) {
             if (uniqueName == m_list[i].uniqueName()) {
                 m_list[i].setEnabled(false);
@@ -481,17 +666,17 @@ void IMPage::Private::moveDownIM()
 
         int i = 0, curIMIdx = -1, nextIMIdx = -1;
         for (i = 0; i < m_list.size(); i ++) {
-            if (curIndex.data(Qt::UserRole) == m_list[i].uniqueName())
+            if (curIndex.data(FcitxIMUniqueNameRole) == m_list[i].uniqueName())
                 curIMIdx = i;
 
-            if (nextIndex.data(Qt::UserRole) == m_list[i].uniqueName())
+            if (nextIndex.data(FcitxIMUniqueNameRole) == m_list[i].uniqueName())
                 nextIMIdx = i;
         }
 
         if (curIMIdx >= 0 && nextIMIdx >= 0 && curIMIdx != nextIMIdx) {
             m_list.swap(curIMIdx, nextIMIdx);
             qStableSort(m_list.begin(), m_list.end());
-            emit updateIMList(m_list, curIndex.data(Qt::UserRole).toString());
+            emit updateIMList(m_list, curIndex.data(FcitxIMUniqueNameRole).toString());
             emit changed();
         }
     }
@@ -506,7 +691,7 @@ void IMPage::Private::configureIM()
     if (!Global::instance()->inputMethodProxy()) {
         return;
     }
-    const QString uniqueName = curIndex.data(Qt::UserRole).toString();
+    const QString uniqueName = curIndex.data(FcitxIMUniqueNameRole).toString();
     QDBusPendingReply< QString > result = Global::instance()->inputMethodProxy()->GetIMAddon(uniqueName);
     result.waitForFinished();
     if (result.isValid()) {
@@ -527,17 +712,17 @@ void IMPage::Private::moveUpIM()
 
         int i = 0, curIMIdx = -1, nextIMIdx = -1;
         for (i = 0; i < m_list.size(); i ++) {
-            if (curIndex.data(Qt::UserRole) == m_list[i].uniqueName())
+            if (curIndex.data(FcitxIMUniqueNameRole) == m_list[i].uniqueName())
                 curIMIdx = i;
 
-            if (nextIndex.data(Qt::UserRole) == m_list[i].uniqueName())
+            if (nextIndex.data(FcitxIMUniqueNameRole) == m_list[i].uniqueName())
                 nextIMIdx = i;
         }
 
         if (curIMIdx >= 0 && nextIMIdx >= 0 && curIMIdx != nextIMIdx) {
             m_list.swap(curIMIdx, nextIMIdx);
             qStableSort(m_list.begin(), m_list.end());
-            emit updateIMList(m_list, curIndex.data(Qt::UserRole).toString());
+            emit updateIMList(m_list, curIndex.data(FcitxIMUniqueNameRole).toString());
             emit changed();
         }
     }
@@ -553,11 +738,6 @@ void IMPage::Private::fetchIMList()
 {
     if (Global::instance()->inputMethodProxy()) {
         m_list = Global::instance()->inputMethodProxy()->iMList();
-        for (int i = 0; i < m_list.size(); i ++) {
-            if (languageIsUnknown(m_list[i].langCode())) {
-                m_list[i].setLangCode("");
-            }
-        }
         qStableSort(m_list.begin(), m_list.end());
         emit updateIMList(m_list, QString());
     }
@@ -586,4 +766,136 @@ IMPage::~IMPage()
     delete m_ui;
 }
 
+
+
+IMDelegate::IMDelegate(QObject* parent) : QStyledItemDelegate(parent)
+{
+}
+
+IMDelegate::~IMDelegate()
+{
+}
+
+const int SPACING = 4;
+
+void IMDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
+{
+    if (index.data(FcitxRowTypeRole).toInt() == IMType) {
+        QStyledItemDelegate::paint(painter, option, index);
+        return;
+    }
+
+    painter->save();
+    painter->setRenderHint(QPainter::Antialiasing);
+
+    const QString category = index.model()->data(index, Qt::DisplayRole).toString();
+    QRect optRect = option.rect;
+    optRect.translate(SPACING, SPACING);
+    optRect.setWidth(optRect.width() - SPACING * 2);
+    optRect.setHeight(optRect.height() - SPACING * 2);
+    QFont font(QApplication::font());
+    font.setBold(true);
+    const QFontMetrics fontMetrics = QFontMetrics(font);
+
+    QColor outlineColor = option.palette.text().color();
+    outlineColor.setAlphaF(0.35);
+
+    //BEGIN: top left corner
+    {
+        painter->save();
+        painter->setPen(outlineColor);
+        const QPointF topLeft(optRect.topLeft());
+        QRectF arc(topLeft, QSizeF(4, 4));
+        arc.translate(0.5, 0.5);
+        painter->drawArc(arc, 1440, 1440);
+        painter->restore();
+    }
+    //END: top left corner
+
+    //BEGIN: left vertical line
+    {
+        QPoint start(optRect.topLeft());
+        start.ry() += 3;
+        QPoint verticalGradBottom(optRect.topLeft());
+        verticalGradBottom.ry() += fontMetrics.height() + 5;
+        QLinearGradient gradient(start, verticalGradBottom);
+        gradient.setColorAt(0, outlineColor);
+        gradient.setColorAt(1, Qt::transparent);
+        painter->fillRect(QRect(start, QSize(1, fontMetrics.height() + 5)), gradient);
+    }
+    //END: left vertical line
+
+    //BEGIN: horizontal line
+    {
+        QPoint start(optRect.topLeft());
+        start.rx() += 3;
+        QPoint horizontalGradTop(optRect.topLeft());
+        horizontalGradTop.rx() += optRect.width() - 6;
+        painter->fillRect(QRect(start, QSize(optRect.width() - 6, 1)), outlineColor);
+    }
+    //END: horizontal line
+
+    //BEGIN: top right corner
+    {
+        painter->save();
+        painter->setPen(outlineColor);
+        QPointF topRight(optRect.topRight());
+        topRight.rx() -= 4;
+        QRectF arc(topRight, QSizeF(4, 4));
+        arc.translate(0.5, 0.5);
+        painter->drawArc(arc, 0, 1440);
+        painter->restore();
+    }
+    //END: top right corner
+
+    //BEGIN: right vertical line
+    {
+        QPoint start(optRect.topRight());
+        start.ry() += 3;
+        QPoint verticalGradBottom(optRect.topRight());
+        verticalGradBottom.ry() += fontMetrics.height() + 5;
+        QLinearGradient gradient(start, verticalGradBottom);
+        gradient.setColorAt(0, outlineColor);
+        gradient.setColorAt(1, Qt::transparent);
+        painter->fillRect(QRect(start, QSize(1, fontMetrics.height() + 5)), gradient);
+    }
+    //END: right vertical line
+
+    //BEGIN: text
+    {
+        QRect textRect(option.rect);
+        textRect.setTop(textRect.top() + 7);
+        textRect.setLeft(textRect.left() + 7);
+        textRect.setHeight(fontMetrics.height());
+        textRect.setRight(textRect.right() - 7);
+
+        painter->save();
+        painter->setFont(font);
+        QColor penColor(option.palette.text().color());
+        penColor.setAlphaF(0.6);
+        painter->setPen(penColor);
+        painter->drawText(textRect, Qt::AlignLeft | Qt::AlignVCenter, category);
+        painter->restore();
+    }
+    //END: text
+
+    painter->restore();
+}
+
+QSize IMDelegate::sizeHint(const QStyleOptionViewItem& option, const QModelIndex& index) const
+{
+    if (index.data(FcitxRowTypeRole).toInt() == IMType) {
+        return QStyledItemDelegate::sizeHint(option, index);
+    }
+    else {
+        QFont font(QApplication::font());
+        font.setBold(true);
+        QFontMetrics fontMetrics(font);
+        const int height = fontMetrics.height() + 1 /* 1 pixel-width gradient */
+                                                + 11 /* top and bottom separation */ + SPACING;
+        return QSize(0, height);
+    }
+}
+
+
 }
